/*
 * @file vle/gvle/global/RvizInsideGVLE/PluginRvizInsideGVLE.cpp
 *
 * This file is part of VLE, a framework for multi-modeling, simulation
 * and analysis of complex dynamical systems
 * http://www.vle-project.org
 *
 * Copyright (c) 2003-2007 Gauthier Quesnel <quesnel@users.sourceforge.net>
 * Copyright (c) 2003-2011 ULCO http://www.univ-littoral.fr
 * Copyright (c) 2007-2011 INRA http://www.inra.fr
 *
 * See the AUTHORS or Authors.txt file for copyright owners and contributors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <vle/gvle/GVLE.hpp>
#include <vle/gvle/GVLEMenuAndToolbar.hpp>
#include <vle/gvle/Message.hpp>
#include <vle/utils/Package.hpp>
#include <gtkmm/widget.h>
#include <gtkmm/menubar.h>
#include <gtkmm/actiongroup.h>
#include <gtkmm/dialog.h>
#include <gtkmm/toolbar.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/builder.h>
#include <gtkmm/stock.h>
#include <vle/vpz/AtomicModel.hpp>
#include <vle/vpz/Condition.hpp>
#include <vle/utils/Spawn.hpp>
#include <iostream>

namespace vle {
namespace gvle {
namespace global {
namespace rinsidegvle {

typedef std::vector < std::string > strings_t;

const Glib::ustring UI_DEFINITION =
    "<ui>"
    "    <menubar name='MenuBar'>"
    "        <menu name='MenuFile' action='MenuFile'>"
    "            <menuitem action='launch Rviz'/>"
    "        </menu>"
    "    </menubar>"
    "</ui>";

/**
 * @class PluginRvizInsideGVLE
 * @brief testing the global plugin
 *
 */


class PluginRvizInsideGVLE : public GlobalPlugin
{
public:

    /**
     * Class constructor.
     *
     * @param[in] package
     * @param[in] library
     * @param[in/out] gvle : an access to everything
     */
    PluginRvizInsideGVLE(const std::string& package,
                 const std::string& library,
                 vle::gvle::GVLE* gvle):
        GlobalPlugin(package, library, gvle)
    {
    }

    /**
     * Class destructor.
     */
    virtual ~PluginRvizInsideGVLE()
    {

        mConnection.disconnect();

        m_refActionGroup->remove(m_refActionGroup->get_action("launch Rviz"));
        m_refUIManager->remove_ui(mMergeId);

    }

    /**
     * Run
     */
    void run()
    {
        m_refUIManager = mGVLE->getMenu()->getUIManager();
        m_refActionGroup = mGVLE->getMenu()->getActionGroup();

        createNewActions();

        updateUI();
    }

    void createNewActions()
    {
        mAction = Gtk::Action::create("launch Rviz", Gtk::Stock::EXECUTE,
                                     _("launch Rviz"), _("launch Rviz"));

        m_refActionGroup->add(mAction, Gtk::AccelKey("<control><shift>g"));
        mConnection = mAction->signal_activate().connect(
            sigc::mem_fun(this, &vle::gvle::global::rinsidegvle::PluginRvizInsideGVLE::launchR));
    }


    void updateUI()
    {
#ifdef GLIBMM_EXCEPTIONS_ENABLED
        try {
            mMergeId = m_refUIManager->add_ui_from_string(UI_DEFINITION);
        } catch(const Glib::Error& ex) {
            std::cerr << "building menus failed: " <<  ex.what();
        }
#else
        std::auto_ptr<Glib::Error> ex;
        mMergeId = m_refUIManager->add_ui_from_string(UI_DEFINITION, ex);
        if(ex.get()) {
            std::cerr << "building menus failed: " <<  ex->what();
        }
#endif //GLIBMM_EXCEPTIONS_ENABLED
    }

    void launchR()
    {
        std::string prg = "/usr/bin/xterm";
        
        std::vector < std::string > args;
        args.push_back("-e");
        args.push_back("R --interactive -q  < launchShinyApp.R");

        utils::Package mPack("RinsideGVLE");       
        
        vle::utils::Spawn& spawn = mGVLE->spawnPool().addSpawn();
        if (spawn.start(prg, mPack.getDir() + "/R/", args)) {
            std::string output, error;
            while (not spawn.isfinish()) {
                std::string output, error;
                spawn.get(&output, &error);
                if (!output.empty() | !error.empty())
                    std::cout << output << " " << error << std::endl;
            }
            std::string msg;
            bool success;
            if (spawn.status(&msg, &success))
                std::cout << ((success) ? "success" : "failed") << std::endl << msg << std::endl;
        } else {
            std::cerr << "failed to start " << prg << std::endl;
        }
    }

    Glib::RefPtr< Gtk::Action > mAction;
    sigc::connection mConnection;

    Glib::RefPtr<Gtk::UIManager>   m_refUIManager;
    Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;

    Gtk::UIManager::ui_merge_id mMergeId;
};

}
}
}
}    // namespace vle gvle global rinsidegvle

DECLARE_GVLE_GLOBALPLUGIN(vle::gvle::global::rinsidegvle::PluginRvizInsideGVLE)
